/*
 *  MySQL driver for OpenDBX
 *  Copyright (C) 2004-2006 Norbert Sendetzky <norbert@linuxnetworks.de>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA.
 */



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/select.h>
#include <mysql.h>
#include "lib/odbx.h"
#include "lib/odbxdrv.h"



#ifndef MYSQLBACKEND_H
#define MYSQLBACKEND_H


/* MySQL 5.0.24 bug workaround */
#ifndef ulong
#	define ulong unsigned long
#endif


/*
 *  Auxillary data structures attached to odbx_t and odbx_result_t
 */

struct myconn
{
	char* host;
	unsigned int port;
	ulong flags;
	char* mode;
	int tls;
	int first;
};

struct myres
{
	MYSQL_ROW row;
	MYSQL_FIELD* fields;
	unsigned long* lengths;
	unsigned long columns;
};



/*
 *  Basic operations
 */

static int mysql_odbx_init( odbx_t* handle, const char* host, const char* port );

static int mysql_odbx_bind( odbx_t* handle, const char* database, const char* who, const char* cred, int method );

static int mysql_odbx_unbind( odbx_t* handle );

static int mysql_odbx_finish( odbx_t* handle );

static int mysql_odbx_get_option( odbx_t* handle, unsigned int option, void* value );

static int mysql_odbx_set_option( odbx_t* handle, unsigned int option, void* value );

static const char* mysql_odbx_error( odbx_t* handle );

static int mysql_odbx_error_type( odbx_t* handle );

static int mysql_odbx_escape( odbx_t* handle, const char* from, unsigned long fromlen, char* to, unsigned long* tolen );

static int mysql_odbx_query( odbx_t* handle, const char* query, unsigned long length );

static int mysql_odbx_result( odbx_t* handle, odbx_result_t** result, struct timeval* timeout, unsigned long chunk );

static void mysql_odbx_result_free( odbx_result_t* result );

static int mysql_odbx_row_fetch( odbx_result_t* result );

static uint64_t mysql_odbx_rows_affected( odbx_result_t* result );

static unsigned long mysql_odbx_column_count( odbx_result_t* result );

static const char* mysql_odbx_column_name( odbx_result_t* result, unsigned long pos );

static int mysql_odbx_column_type( odbx_result_t* result, unsigned long pos );

static unsigned long mysql_odbx_field_length( odbx_result_t* result, unsigned long pos );

static const char* mysql_odbx_field_value( odbx_result_t* result, unsigned long pos );



/*
 * MySQL private functions
 */

static int mysql_priv_setmode( odbx_t* handle, const char* mode );



#endif
