/*
 *  PostgreSQL driver for OpenDBX
 *  Copyright (C) 2004-2006 Norbert Sendetzky <norbert@linuxnetworks.de>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA.
 */




#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <sys/select.h>
#include <libpq-fe.h>
#include "lib/odbx.h"
#include "lib/odbxdrv.h"



#ifndef PGSQLBACKEND_H
#define PGSQLBACKEND_H



#define PGSQL_BUFLEN 511



/*
 *  PostgreSQL OID values (from catalog/pg_type.h)
 */

#define BOOLOID			16
#define BYTEAOID		17
#define CHAROID			18
#define INT8OID			20
#define INT2OID			21
#define INT4OID			23
#define TEXTOID			25
#define FLOAT4OID		700
#define FLOAT8OID		701
#define BPCHAROID		1042
#define VARCHAROID	1043
#define DATEOID			1082
#define TIMEOID			1083
#define TIMESTAMPOID	1114
#define TIMESTAMPTZOID	1184
#define INTERVALOID	1186
#define TIMETZOID		1266
#define NUMERICOID	1700
#define ANYARRAYOID	2277



/*
 *  Auxillary data structures attached to odbx_t and odbx_result_t
 */

struct pgconn
{
	char info[PGSQL_BUFLEN+1];
	int infolen;
	int ssl;
	int errtype;
};

struct pgres
{
	int count;
	int total;
};



/*
 *  Basic operations
 */

static int pgsql_odbx_init( odbx_t* handle, const char* host, const char* port );

static int pgsql_odbx_bind( odbx_t* handle, const char* database, const char* who, const char* cred, int method );

static int pgsql_odbx_unbind( odbx_t* handle );

static int pgsql_odbx_finish( odbx_t* handle );

static int pgsql_odbx_get_option( odbx_t* handle, unsigned int option, void* value );

static int pgsql_odbx_set_option( odbx_t* handle, unsigned int option, void* value );

static const char* pgsql_odbx_error( odbx_t* handle );

static int pgsql_odbx_error_type( odbx_t* handle );

static int pgsql_odbx_escape( odbx_t* handle, const char* from, unsigned long fromlen, char* to, unsigned long* tolen );

static int pgsql_odbx_query( odbx_t* handle, const char* query, unsigned long length );

static int pgsql_odbx_result( odbx_t* handle, odbx_result_t** result, struct timeval* timeout, unsigned long chunk );

static void pgsql_odbx_result_free( odbx_result_t* result );

static int pgsql_odbx_row_fetch( odbx_result_t* result );

static uint64_t pgsql_odbx_rows_affected( odbx_result_t* result );

static unsigned long pgsql_odbx_column_count( odbx_result_t* result );

static const char* pgsql_odbx_column_name( odbx_result_t* result, unsigned long pos );

static int pgsql_odbx_column_type( odbx_result_t* result, unsigned long pos );

static unsigned long pgsql_odbx_field_length( odbx_result_t* result, unsigned long pos );

static const char* pgsql_odbx_field_value( odbx_result_t* result, unsigned long pos );



#endif
