/*
 *  SQLite3 driver for OpenDBX
 *  Copyright (C) 2005 Norbert Sendetzky <norbert@linuxnetworks.de>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA.
 */




#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/select.h>
#include <sqlite3.h>
#include "lib/odbx.h"
#include "lib/odbxdrv.h"



#ifndef SQLITE3BACKEND_H
#define SQLITE3BACKEND_H



/*
 *  Auxillary data structures attached to odbx_t and odbx_result_t
 */

struct sconn
{
	char* path;
	int pathlen;
	char* stmt;
	char* tail;
	unsigned long length;
};



/*
 *  Basic operations
 */

static int sqlite3_odbx_init( odbx_t* handle, const char* host, const char* port );

static int sqlite3_odbx_bind( odbx_t* handle, const char* database, const char* who, const char* cred, int method );

static int sqlite3_odbx_unbind( odbx_t* handle );

static int sqlite3_odbx_finish( odbx_t* handle );

static int sqlite3_odbx_get_option( odbx_t* handle, unsigned int option, void* value );

static int sqlite3_odbx_set_option( odbx_t* handle, unsigned int option, void* value );

static const char* sqlite3_odbx_error( odbx_t* handle );

static int sqlite3_odbx_error_type( odbx_t* handle );

static int sqlite3_odbx_query( odbx_t* handle, const char* query, unsigned long length );

static int sqlite3_odbx_result( odbx_t* handle, odbx_result_t** result, struct timeval* timeout, unsigned long chunk );

static void sqlite3_odbx_result_free( odbx_result_t* result );

static int sqlite3_odbx_row_fetch( odbx_result_t* result );

static uint64_t sqlite3_odbx_rows_affected( odbx_result_t* result );

static unsigned long sqlite3_odbx_column_count( odbx_result_t* result );

static const char* sqlite3_odbx_column_name( odbx_result_t* result, unsigned long pos );

static int sqlite3_odbx_column_type( odbx_result_t* result, unsigned long pos );

static unsigned long sqlite3_odbx_field_length( odbx_result_t* result, unsigned long pos );

static const char* sqlite3_odbx_field_value( odbx_result_t* result, unsigned long pos );



#endif
