/*
 *  Sybase driver for OpenDBX
 *  Copyright (C) 2006 Norbert Sendetzky <norbert@linuxnetworks.de>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307 USA.
 */



#include <cspublic.h>
#include <ctpublic.h>
#include "lib/odbx.h"
#include "lib/odbxdrv.h"



#ifndef SYBASEBACKEND_H
#define SYBASEBACKEND_H



#define SYBASE_ERRLEN 512



/*
 *  Auxillary data structures attached to odbx_t and odbx_result_t
 */

struct sybconn
{
	char* host;
	size_t hostlen;
	CS_CONTEXT* ctx;
	CS_CONNECTION* conn;
	int errtype;
	char errmsg[SYBASE_ERRLEN];
};

struct sybres
{
	CS_CHAR* value;
	CS_INT length;
	CS_SMALLINT status;
};

struct sybares
{
	CS_INT cols;
	CS_DATAFMT* fmt;
};



/*
 *  Basic operations
 */

static int sybase_odbx_init( odbx_t* handle, const char* host, const char* port );

static int sybase_odbx_bind( odbx_t* handle, const char* database, const char* who, const char* cred, int method );

static int sybase_odbx_unbind( odbx_t* handle );

static int sybase_odbx_finish( odbx_t* handle );

static int sybase_odbx_get_option( odbx_t* handle, unsigned int option, void* value );

static int sybase_odbx_set_option( odbx_t* handle, unsigned int option, void* value );

static const char* sybase_odbx_error( odbx_t* handle );

static int sybase_odbx_error_type( odbx_t* handle );

static int sybase_odbx_query( odbx_t* handle, const char* query, unsigned long length );

static int sybase_odbx_result( odbx_t* handle, odbx_result_t** result, struct timeval* timeout, unsigned long chunk );

static void sybase_odbx_result_free( odbx_result_t* result );

static int sybase_odbx_row_fetch( odbx_result_t* result );

static uint64_t sybase_odbx_rows_affected( odbx_result_t* result );

static unsigned long sybase_odbx_column_count( odbx_result_t* result );

static const char* sybase_odbx_column_name( odbx_result_t* result, unsigned long pos );

static int sybase_odbx_column_type( odbx_result_t* result, unsigned long pos );

static unsigned long sybase_odbx_field_length( odbx_result_t* result, unsigned long pos );

static const char* sybase_odbx_field_value( odbx_result_t* result, unsigned long pos );



/*
 *  Private functions
 */

static int sybase_priv_init( struct sybconn* aux );

static int sybase_priv_cleanup( odbx_t* handle );

static int sybase_priv_convert( odbx_result_t* result );

static CS_INT sybase_priv_collength( CS_DATAFMT* column );

static CS_RETCODE CS_PUBLIC sybase_priv_csmsg_handler( CS_CONTEXT* ctx, CS_CLIENTMSG* msg );

static CS_RETCODE CS_PUBLIC sybase_priv_ctmsg_handler( CS_CONTEXT* ctx, CS_CONNECTION* conn, CS_CLIENTMSG* msg );

static CS_RETCODE CS_PUBLIC sybase_priv_svmsg_handler( CS_CONTEXT* ctx, CS_CONNECTION* conn, CS_SERVERMSG* msg );



#endif
