#include "lib/odbxlib.h"
#include <string.h>
#include <stdio.h>



#if defined( ODBX_STATIC )


#if defined( HAVE_LIBFBCLIENT )
extern struct odbx_ops firebird_odbx_ops;
#endif
#if defined( HAVE_LIBSYBDB )
extern struct odbx_ops mssql_odbx_ops;
#endif
#if defined( HAVE_LIBMYSQLCLIENT_R ) || defined( HAVE_LIBMYSQLCLIENT )
extern struct odbx_ops mysql_odbx_ops;
#endif
#if defined( HAVE_LIBCLNTSH )
extern struct odbx_ops oracle_odbx_ops;
#endif
#if defined( HAVE_LIBPQ )
extern struct odbx_ops pgsql_odbx_ops;
#endif
#if defined( HAVE_LIBSQLITE )
extern struct odbx_ops sqlite_odbx_ops;
#endif
#if defined( HAVE_LIBDQLITE3 )
extern struct odbx_ops sqlite3_odbx_ops;
#endif
#if defined( HAVE_LIBCT ) || defined( HAVE_LIBSYBCT_R ) || defined( HAVE_LIBSYBCT )
extern struct odbx_ops sybase_odbx_ops;
#endif


struct odbx_backend_syms
{
	char* name;
	struct odbx_ops* ops;
};


struct odbx_backend_syms odbx_lib_ops[] = {
#if defined( HAVE_LIBMYSQLCLIENT_R ) || defined( HAVE_LIBMYSQLCLIENT )
	{ "mysql", &mysql_odbx_ops },
#endif
#if defined( HAVE_LIBPQ )
	{ "pgsql", &pgsql_odbx_ops },
#endif
#if defined( HAVE_LIBDQLITE3 )
	{ "sqlite3", &sqlite3_odbx_ops },
#endif
#if defined( HAVE_LIBFBCLIENT )
	{ "firebird", &firebird_odbx_ops },
#endif
#if defined( HAVE_LIBCLNTSH )
	{ "oracle", &oracle_odbx_ops },
#endif
#if defined( HAVE_LIBSYBDB )
	{ "mssql", &mssql_odbx_ops },
#endif
#if defined( HAVE_LIBCT ) || defined( HAVE_LIBSYBCT_R ) || defined( HAVE_LIBSYBCT )
	{ "sybase", &sybase_odbx_ops },
#endif
#if defined( HAVE_LIBSQLITE )
	{ "sqlite", &sqlite_odbx_ops },
#endif
	{ NULL, NULL }
};



int _odbx_lib_open( struct odbx_t* handle, const char* backend )
{
	struct odbx_backend_syms* current = odbx_lib_ops;

	while( current->name != NULL )
	{
		if( strcmp( current->name, backend ) == 0 )
		{
			handle->ops = current->ops;
			return ODBX_ERR_SUCCESS;
		}

		current++;
	}

	return -ODBX_ERR_NOTEXIST;
}



int _odbx_lib_close( struct odbx_t* handle )
{
	return ODBX_ERR_SUCCESS;
}



#elif defined( HAVE_LIBDL )


#ifdef HAVE_DLFCN_H
#include <dlfcn.h>
#endif



int _odbx_lib_open( struct odbx_t* handle, const char* backend )
{
	size_t len, plen;
	char lib[ODBX_PATHSIZE+1];
	void (*odbxreg)(struct odbx_ops**);


	len = plen = snprintf( lib, ODBX_PATHSIZE, LIBPATH );
	len += snprintf( lib + len, ODBX_PATHSIZE, "lib" );
	len += snprintf( lib + len, ODBX_PATHSIZE - len, backend );
	len += snprintf( lib + len, ODBX_PATHSIZE - len, "backend.so" );

	if( len > ODBX_PATHSIZE )
	{
		return -ODBX_ERR_SIZE;
	}
	lib[len] = '\0';

	if( ( handle->backend = dlopen( lib, RTLD_LAZY ) ) == NULL )
	{
		fprintf( stderr, "_odbx_lib_open(): %s: %s - %s\n", gettext_noop("Loading backend library failed"), lib, dlerror() );

		if( ( handle->backend = dlopen( lib + plen, RTLD_LAZY ) ) == NULL )
		{
			fprintf( stderr, "_odbx_lib_open(): %s: %s - %s\n", gettext_noop("Loading backend library failed"), lib + plen, dlerror() );
			return -ODBX_ERR_NOTEXIST;
		}
	}

	if( ( *(void **) (&odbxreg) = dlsym( handle->backend, "odbxdrv_register" ) ) == NULL )
	{
		return -ODBX_ERR_NOOP;
	}

	(*odbxreg)( &(handle->ops) );
	return ODBX_ERR_SUCCESS;
}



int _odbx_lib_close( struct odbx_t* handle )
{
	if( dlclose( handle->backend ) != 0 )
	{
		return ODBX_ERR_PARAM;
	}

	return ODBX_ERR_SUCCESS;
}



#elif defined( WIN32 ) && defined ( DLL_EXPORT )


#ifdef HAVE_WINDOWS_H
#include <windows.h>
#endif



int _odbx_lib_open( struct odbx_t* handle, const char* backend )
{
	size_t len, plen;
	char lib[ODBX_PATHSIZE+1];
	void (*odbxreg)(struct odbx_ops**);


	len = plen = snprintf( lib, ODBX_PATHSIZE, LIBPATH );
	len += snprintf( lib + len, ODBX_PATHSIZE, "lib" );
	len += snprintf( lib + len, ODBX_PATHSIZE - len, backend );
	len += snprintf( lib + len, ODBX_PATHSIZE - len, "backend.dll" );

	if( len > ODBX_PATHSIZE )
	{
		return -ODBX_ERR_SIZE;
	}
	lib[len] = '\0';

	if( ( handle->backend = (void*) LoadLibrary( lib ) ) == NULL )
	{
		if( ( handle->backend = (void*) LoadLibrary( lib + plen ) ) == NULL )
		{
			return -ODBX_ERR_NOTEXIST;
		}
	}

	if( ( *(void **) (&odbxreg) = GetProcAddress( (HMODULE) handle->backend, "odbxdrv_register" ) ) == NULL )
	{
		return -ODBX_ERR_NOOP;
	}

	(*odbxreg)( &(handle->ops) );
	return ODBX_ERR_SUCCESS;
}



int _odbx_lib_close( struct odbx_t* handle )
{
	if( FreeLibrary( (HMODULE) handle->backend ) != 0 )
	{
		return ODBX_ERR_PARAM;
	}

	return ODBX_ERR_SUCCESS;
}



#else
#error "Building shared libraries requires capabilities to load libraries dynamically"
#endif
